import 'dart:async';
import 'dart:io';

import 'package:eyflutter_core/device/android_device_info.dart';
import 'package:eyflutter_core/device/device_channels.dart';
import 'package:eyflutter_core/device/device_info.dart';
import 'package:eyflutter_core/device/ios_device_info.dart';
import 'package:eyflutter_core/kit/timer/system.dart';
import 'package:eyflutter_core/kit/utils/json_utils.dart';
import 'package:eyflutter_core/kit/utils/string/json_extension.dart';
import 'package:eyflutter_core/kit/utils/string/string_extension.dart';
import 'package:eyflutter_core/mq/cloud_channel_manager.dart';
import 'package:eyflutter_core/storage/mmkv_utils.dart';

class DeviceUtils {
  factory DeviceUtils() => _getInstance();

  static DeviceUtils get instance => _getInstance();
  static DeviceUtils? _instance;

  static DeviceUtils _getInstance() {
    return _instance ??= new DeviceUtils._internal();
  }

  DeviceUtils._internal();

  String _deviceMethodName = "1c94d6dafa7e7903";
  String _deviceCacheKey = "ec84902b39acefa2";
  String _deviceId = "";
  AndroidDeviceInfo? _androidDeviceInfo;
  IosDeviceInfo? _iosDeviceInfo;
  String _cacheDeviceData = "";

  //设备缓存信息是否在获取中
  bool _cacheDeviceDataTaking = false;

  //上一次获取缓存时间
  int _preTakeCacheTime = 0;

  String _getDeviceId() {
    if (_deviceId.isNotEmptyString) {
      return _deviceId;
    }
    _deviceId = deviceInfo.deviceId;
    return _deviceId;
  }

  /// 设备id
  String get deviceId => _getDeviceId();

  /// 获取设备信息
  DeviceInfo get deviceInfo => _getDeviceInfo();

  /// 获取设备信息
  Future<DeviceInfo> get device => _getAsyncDeviceInfo();

  DeviceInfo _getDeviceInfo() {
    if (Platform.isAndroid) {
      var deviceInfo = _getAndroidCacheDevice();
      return _toDeviceInfoWithAndroid(deviceInfo);
    } else if (Platform.isIOS) {
      var deviceInfo = _getIOSCacheDevice();
      return _toDeviceInfoWithIos(deviceInfo);
    } else {
      return DeviceInfo();
    }
  }

  Future<DeviceInfo> _getAsyncDeviceInfo() async {
    if (Platform.isAndroid) {
      var deviceInfo = await _getAsyncAndroidCacheDevice();
      return Future.value(_toDeviceInfoWithAndroid(deviceInfo));
    } else if (Platform.isIOS) {
      var deviceInfo = await _getAsyncIOSCacheDevice();
      return Future.value(_toDeviceInfoWithIos(deviceInfo));
    } else {
      return Future.value(DeviceInfo());
    }
  }

  DeviceInfo _toDeviceInfoWithIos(IosDeviceInfo? iosDeviceInfo) {
    var deviceInfo = DeviceInfo();
    deviceInfo.deviceId = iosDeviceInfo?.deviceId ?? "";
    deviceInfo.model = iosDeviceInfo?.model ?? "";
    deviceInfo.brand = iosDeviceInfo?.brand ?? "";
    deviceInfo.density = iosDeviceInfo?.density ?? "";
    deviceInfo.display = iosDeviceInfo?.resolution ?? "";
    deviceInfo.deviceType = 2;
    deviceInfo.sys = "${iosDeviceInfo?.systemName ?? ""} ${iosDeviceInfo?.sysVersion ?? ""}";
    return deviceInfo;
  }

  DeviceInfo _toDeviceInfoWithAndroid(AndroidDeviceInfo? androidDeviceInfo) {
    var deviceInfo = DeviceInfo();
    deviceInfo.deviceId = androidDeviceInfo?.deviceId ?? "";
    deviceInfo.display = androidDeviceInfo?.resolution ?? "";
    deviceInfo.hardware = androidDeviceInfo?.hardware ?? "";
    deviceInfo.deviceType = 1;
    deviceInfo.density = androidDeviceInfo?.density ?? "";
    deviceInfo.board = androidDeviceInfo?.board ?? "";
    deviceInfo.brand = androidDeviceInfo?.brand ?? "";
    deviceInfo.abis = androidDeviceInfo?.abis ?? "";
    deviceInfo.model = androidDeviceInfo?.model ?? "";
    deviceInfo.host = androidDeviceInfo?.host ?? "";
    deviceInfo.manufacturer = androidDeviceInfo?.manufacturer ?? "";
    deviceInfo.sys = "${androidDeviceInfo?.deviceName ?? ""} ${androidDeviceInfo?.sysVersion ?? ""}";
    Map<String, String> extensionMap = {
      "androidId": androidDeviceInfo?.androidId ?? "",
      "cpuCoreNum": androidDeviceInfo?.cpuCoreNum ?? "",
      "kernelVersion": androidDeviceInfo?.kernelVersion ?? "",
      "runMemory": androidDeviceInfo?.runMemory ?? "",
      "storageSize": androidDeviceInfo?.storageSize ?? ""
    };
    deviceInfo.extension = JsonUtils.toJson(extensionMap);
    return deviceInfo;
  }

  AndroidDeviceInfo? _getAndroidCacheDevice() {
    if (_androidDeviceInfo == null || (_androidDeviceInfo?.deviceId.isEmptyString ?? true)) {
      var deviceData = _getDeviceData();
      if (!deviceData.isEmptyJson) {
        var map = JsonUtils.fromJson(deviceData);
        _androidDeviceInfo = AndroidDeviceInfo.fromJson(map);
        return _androidDeviceInfo;
      } else {
        return null;
      }
    } else {
      return _androidDeviceInfo;
    }
  }

  Future<AndroidDeviceInfo?> _getAsyncAndroidCacheDevice() async {
    if (_androidDeviceInfo == null && (_androidDeviceInfo?.deviceId.isEmptyString ?? true)) {
      var deviceData = await _getAsyncDeviceData();
      if (!deviceData.isEmptyJson) {
        var map = JsonUtils.fromJson(deviceData);
        _androidDeviceInfo = AndroidDeviceInfo.fromJson(map);
        return _androidDeviceInfo;
      } else {
        return null;
      }
    } else {
      return Future.value(_androidDeviceInfo);
    }
  }

  //是否在模拟器中运行
  Future<bool?> get isSimulator => DeviceChannels.isSimulator;

  //设备是否被root过
  Future<bool?> get isRoot => DeviceChannels.isRoot;

  //是否超出提取时间
  bool get _isTakeTimeOut {
    int diff = System.currentTimeMillis - _preTakeCacheTime;
    return diff >= 30000;
  }

  String _getDeviceData() {
    if (_cacheDeviceDataTaking && !_isTakeTimeOut) {
      return _cacheDeviceData;
    }
    _cacheDeviceDataTaking = true;
    _preTakeCacheTime = System.currentTimeMillis;
    MmkvUtils.instance.getString(_deviceCacheKey, part: MMkvPart.normal).then((data) {
      if (data.isEmptyString) {
        CloudChannelManager.instance.send<String>(_deviceMethodName).then((response) {
          _cacheDeviceDataTaking = false;
          if (response.isNotEmptyString) {
            _cacheDeviceData = response!;
            MmkvUtils.instance.putString(_deviceCacheKey, part: MMkvPart.normal, value: response);
          }
        });
      } else {
        _cacheDeviceData = data ?? "";
        _cacheDeviceDataTaking = false;
      }
    });
    return _cacheDeviceData;
  }

  Future<String> _getAsyncDeviceData() async {
    if (_cacheDeviceData.isNotEmptyString) {
      return Future.value(_cacheDeviceData);
    }
    var data = await MmkvUtils.instance.getString(_deviceCacheKey, part: MMkvPart.normal);
    if (data.isNotEmptyString) {
      _cacheDeviceData = data ?? "";
      return Future.value(_cacheDeviceData);
    }
    var deviceData = await CloudChannelManager.instance.send<String>(_deviceMethodName);
    _cacheDeviceData = deviceData ?? "";
    if (deviceData.isNotEmptyString) {
      MmkvUtils.instance.putString(_deviceCacheKey, part: MMkvPart.normal, value: _cacheDeviceData);
    }
    return Future.value(_cacheDeviceData);
  }

  IosDeviceInfo? _getIOSCacheDevice() {
    if (_iosDeviceInfo == null && (_iosDeviceInfo?.deviceId.isEmptyString ?? true)) {
      var deviceData = _getDeviceData();
      if (!deviceData.isEmptyJson) {
        var map = JsonUtils.fromJson(deviceData);
        _iosDeviceInfo = IosDeviceInfo.fromJson(map);
        return _iosDeviceInfo;
      } else {
        return null;
      }
    } else {
      return _iosDeviceInfo;
    }
  }

  Future<IosDeviceInfo?> _getAsyncIOSCacheDevice() async {
    if (_iosDeviceInfo == null && (_iosDeviceInfo?.deviceId.isEmptyString ?? true)) {
      var deviceData = await _getAsyncDeviceData();
      if (!deviceData.isEmptyJson) {
        var map = JsonUtils.fromJson(deviceData);
        _iosDeviceInfo = IosDeviceInfo.fromJson(map);
        return _iosDeviceInfo;
      } else {
        return null;
      }
    } else {
      return Future.value(_iosDeviceInfo);
    }
  }
}
